from queue import Queue
from threading import Thread
from typing import List

from .models import PricesList
from .sources.base import SourceBase


class SourceManager:
    _sources: List[SourceBase] = []

    def __init__(self) -> None:
        pass

    def add(self, source: SourceBase) -> None:
        if source not in self._sources:
            self._sources.append(source)

    def get_prices_list(self) -> PricesList:
        queue = Queue()
        threads: List[Thread] = []

        for source in self._sources:
            tr = Thread(target=source.get_price, args=(queue,))
            threads.append(tr)
            tr.start()

        for tr in threads:
            tr.join()

        prices = []
        while not queue.empty():
            prices.append(queue.get())

        return PricesList(prices)



import logging
from queue import Queue
from typing import Tuple

from ..exceptions import TetherException
from ..models import Price

logging.basicConfig(format='[%(levelname)s] %(asctime)s - %(message)s', level=logging.ERROR)
logger = logging.getLogger(__name__)
logger.setLevel(logging.ERROR)


class SourceBase:
    _source_name = ''

    def get_price(self, queue: Queue) -> None:
        try:
            buy, sell = self._fetch_price()
            queue.put(
                Price(self._source_name, buy, sell)
            )
        except TetherException as err:
            self.log_error(err)

    def log_error(self, error: Exception) -> None:
        message = f'[{self._source_name}] - Something went wrong: %s'
        logger.log(logging.ERROR, message, error)

    def _fetch_price(self) -> Tuple[int, int]:
        raise NotImplementedError



import json
from typing import Tuple

import requests
from requests import RequestException

from .base import SourceBase
from .. import config
from ..exceptions import TetherException


class Bitbarg(SourceBase):
    _source_name = 'Bitbarg'
    __site_url = 'https://bitbarg.com/live-price/'

    def _fetch_price(self) -> Tuple[int, int]:
        try:
            req = requests.get(self.__site_url, timeout=config.TIMEOUT)
        except RequestException:
            raise TetherException('Cannot connect to Bitbarg sever.')

        text = req.text
        start = text.find('prices')
        end = start + 1
        while text[end] != '}':
            end += 1

        try:
            prices = json.loads(text[start + 8:end + 1])
            return prices['buy'], prices['sell']
        except json.JSONDecodeError:
            raise TetherException('Cannot decode response text.')



from json import JSONDecodeError
from typing import Tuple

import requests
from requests import RequestException

from .base import SourceBase
from .. import config
from ..exceptions import TetherException


class ArzPaya(SourceBase):
    _source_name = 'ArzPaya'
    __site_url = 'https://api.arzpaya.com/public/Landing/usdt'

    def _fetch_price(self) -> Tuple[int, int]:
        try:
            req = requests.get(self.__site_url, timeout=config.TIMEOUT)
        except RequestException:
            raise TetherException('Cannot connect to ArzPaya sever.')

        try:
            data = req.json()['Data']
            buy = int(data['CurrentBuyPrice'])
            sell = int(data['CurrentSellPrice'])
            return buy, sell
        except JSONDecodeError:
            raise TetherException('Cannot decode response text.')
